@@ -17,6 +17,7 @@
  */
 
 #include "juice.h"
+#include "addr.h"
 #include "agent.h"
 #include "ice.h"
 
@@ -137,12 +138,10 @@ JUICE_EXPORT int juice_get_selected_addresses(juice_agent_t *agent, char *local,
 	if (agent_get_selected_candidate_pair(agent, &local_cand, &remote_cand))
 		return JUICE_ERR_NOT_AVAIL;
 
-	if (local_size &&
-	    snprintf(local, local_size, "%s:%s", local_cand.hostname, local_cand.service) < 0)
+	if (local_size && addr_record_to_string(&local_cand.resolved, local, local_size) < 0)
 		return JUICE_ERR_FAILED;
 
-	if (remote_size &&
-	    snprintf(remote, remote_size, "%s:%s", remote_cand.hostname, remote_cand.service) < 0)
+	if (remote_size && addr_record_to_string(&remote_cand.resolved, remote, remote_size) < 0)
 		return JUICE_ERR_FAILED;
 
 	return JUICE_ERR_SUCCESS;
