@@ -284,7 +284,9 @@ void server_destroy(juice_server_t *server) {
 
 uint16_t server_get_port(juice_server_t *server) {
 	mutex_lock(&server->mutex);
-	return server->config.port; // updated at creation
+	uint16_t port = server->config.port; // updated at creation
+	mutex_unlock(&server->mutex);
+	return port;
 }
 
 void server_run(juice_server_t *server) {
@@ -859,14 +861,14 @@ int server_process_turn_allocate(juice_server_t *server, const stun_message_t *m
 			JLOG_ERROR("No advertisable relayed address found");
 			goto error;
 		}
-	}
 
-	if (JLOG_INFO_ENABLED) {
-		char src_str[ADDR_MAX_STRING_LEN];
-		addr_record_to_string(src, src_str, ADDR_MAX_STRING_LEN);
-		char relayed_str[ADDR_MAX_STRING_LEN];
-		addr_record_to_string(relayed, relayed_str, ADDR_MAX_STRING_LEN);
-		JLOG_INFO("Allocated TURN relayed address %s for client %s", relayed_str, src_str);
+		if (JLOG_INFO_ENABLED) {
+			char src_str[ADDR_MAX_STRING_LEN];
+			addr_record_to_string(src, src_str, ADDR_MAX_STRING_LEN);
+			char relayed_str[ADDR_MAX_STRING_LEN];
+			addr_record_to_string(relayed, relayed_str, ADDR_MAX_STRING_LEN);
+			JLOG_INFO("Allocated TURN relayed address %s for client %s", relayed_str, src_str);
+		}
 	}
 
 	stun_message_t ans;
