@@ -29,8 +29,6 @@
 #include <functional>
 #include <map>
 #include <mutex>
-#include <shared_mutex>
-#include <unordered_set>
 
 #include "usrsctp.h"
 
